(defun profile-empty-regexps (&optional buffer)
  "Report some matching times for several regular expressions."
  (interactive)
  (unless buffer
    (setq buffer (find-file-noselect (locate-library "regexp-opt"))))
  (with-output-to-temp-buffer "Profiling"
    (princ (profile-one-regexp "a\\`" buffer))        (princ "\n")
    (princ (profile-one-regexp "\\'a" buffer))        (princ "\n")
    (princ (profile-one-regexp "\\=.\\=" buffer))     (princ "\n")
    (princ (profile-one-regexp "\\=a\\=" buffer))     (princ "\n")
    (princ (profile-one-regexp "\\." buffer))         (princ "\n")
    ))

(defun profile-one-regexp (regexp buffer &optional nbrepeats)
  ;; The workhorse
  (setq nbrepeats (or nbrepeats 50000))
  (let (start-time duration1 duration2 found)
    (with-current-buffer buffer
      (save-excursion
	(setq start-time (current-time))
	(goto-char (point-min))
	(dotimes (_ nbrepeats)
	  (looking-at regexp))
	(setq duration1 (time-subtract (current-time) start-time))
	(setq start-time (current-time))
	(goto-char (point-min))
	(dotimes (_ nbrepeats)
	  (when (re-search-forward regexp nil t) 
	    (setq found t)
	    (goto-char (point-min))))	;; return to test position
	(setq duration2 (time-subtract (current-time) start-time))))
    (format "Testing regexp %s %d times\n\tmatch at point-min: %.4fs\n\tsearch in buffer %s (size %d): %.4fs\n%s" regexp nbrepeats (float-time duration1) (buffer-name buffer) 
	    (buffer-size buffer) (float-time duration2)
	    (if found "\t*** WARNING *** a match was found\n" ""))))

